classdef janis_gpml < janis_multiClassWrapper
    % Wrapper class for classifier:
    %      Gaussian process classifier
    % see documentation:
    %      <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
    %
    % for multiple classes: uses one-vs-all method
    
    properties
        multiClassMethod = 'onevsall';      % method for multiple classes
    end
    
    properties (Constant)
        availableMeanFunctions = {  'meanConst'; ...
            'meanLinear'; ...
            'meanMask'; ...
            'meanOne'; ...
            'meanPow'; ...
            'meanProd'; ...
            'meanScale'; ...
            'meanSum';
            'meanZero'};            % available Mean Function, @see <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
        availableCovFunctions = {   'covADD'; ...
            'covFITC';...
            'covLIN';...
            'covMask';...
            'covNNone';...
            'covPeriodic';...
            'covPPiso';...
            'covRQard';...
            'covScale';...
            'covSEiso'; ...
            'covSEard'; ...
            'covSum'};              % available Covariance Function, @see <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
        availableLikFunctions = {   'likErf';...
            'likGauss';...
            'likLaplace';...
            'likLogistic';...
            'likSech2';...
            'likT.m'};              % available Liklehood Function, @see <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
    end
    properties (SetAccess = protected)
        hyp = struct;               % Hyperparameters struct
        meanfunc = @meanZero;       % Mean Function, @see <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
        covfunc =  @covLIN;         % Covariance Function, @see <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
        likfunc = @likErf;          % Liklehood Function, @see <http://www.gaussianprocess.org/gpml/code/matlab/doc/ GPML>
        labelMapping;               % Mapped labels (since gpml only accepts -1 and 1)
    end
    
    methods (Static)
        function shortName = name
            shortName = 'GPML (Gaussian Process Classifier)';
        end
        function    [state msg] = checkSetup
            if exist('gp','file')
                state = true;
                msg = 'GPML files: ✓';
            else
                state = false;
                msg = 'Gaussian Process Classifiers not found. If you would like to use gpml, download it from thttp://www.gaussianprocess.org and add folder to matlab search path.';
            end
        end
        
    end
    
    methods
        function this = binaryTrain(this,trainPatterns,trainLabels)
            % setup
            this.hyp = struct;
            this.hyp.mean = [];
            this.hyp.cov = [];
            % gpml only accpets labels -1 and 1 - therefore map labels, and
            % save it for predcition
            this.labelMapping = sort(unique(trainLabels));
            newLabels = zeros(numel(trainLabels),1);
            newLabels(trainLabels==this.labelMapping(1)) = -1;
            newLabels(trainLabels==this.labelMapping(2)) =  1;
            % train
            this.hyp = minimize(this.hyp, @gp, -100, @infEP, this.meanfunc, this.covfunc, this.likfunc, trainPatterns', newLabels);
            this.classifierStruct.trainPatterns = trainPatterns';
            this.classifierStruct.trainLabels = newLabels;
        end
        
        function [predictions decFactors] = binaryTest(this,testPatterns)
            [a b c d lp] = gp(this.hyp, @infEP, this.meanfunc, this.covfunc, this.likfunc, ...
                this.classifierStruct.trainPatterns, this.classifierStruct.trainLabels, ...
                testPatterns', ones(length(testPatterns), 1));
            % assign all, which are greater than 0 to pos. group
            % and the rest to negative group
            predictions = zeros(numel(a),1);
            predictions(a<0) = this.labelMapping(1);
            predictions(a>=0) = this.labelMapping(2);
            decFactors.decVals = a*(-1);
        end

        % Not implemented
        function classifierStruct  = getClassifier(this,trainPatterns,trainLabels)
            disp('not implemented');
        end
        
        function this = uiconfigure(this)
            mIdx = listdlg('PromptString','Select Mean Function:','SelectionMode','single','ListString',this.availableMeanFunctions);
            cIdx = listdlg('PromptString','Select Covariance Function:','SelectionMode','single','ListString',this.availableCovFunctions);
            lIdx = listdlg('PromptString','Select Liklehodd Function:','SelectionMode','single','ListString',this.availableLikFunctions);
            mStr = ['@' this.availableMeanFunctions{mIdx}];
            cStr = ['@' this.availableCovFunctions{cIdx}];
            lStr = ['@' this.availableLikFunctions{lIdx}];
            this.meanfunc = eval(mStr);
            this.covfunc = eval(cStr);
            this.likfunc = eval(lStr);
        end
        
        function show(this)
            show@janis_multiClassWrapper(this);
            fprintf('Mean Function:        %s\n', func2str(this.meanfunc));
            fprintf('Covariance Function:  %s\n', func2str(this.covfunc));
            fprintf('Liklehood Function:   %s\n', func2str(this.likfunc));
        end
        
        function new = binaryCopy(this)
            new = janis_gpml;
            new.hyp = this.hyp;
            new.meanfunc = this.meanfunc;
            new.covfunc = this.covfunc;
            new.likfunc = this.likfunc;
            new.isBinary = 1;
            new.labelMapping = this.labelMapping;
        end
        
    end
end