classdef janis_multiClassWrapper < janis_classifier
    % Wrapper for binary classifiers, to use simple multi-class-methods
    properties
        isBinary;                       % returns true, if classfier is binary.
    end
    
    properties (Constant)
        availableMultiClassMethods = {'onevsall','onevsone','native'}; % available multiclass methods
    end
    
    properties (Abstract)
        multiClassMethod                % multiclass method to use
    end
    
    methods (Abstract)
        % prediction of binary trained classifiers
        [predictions decFactors] = binaryTest(this,testPatterns)
        % binary training, with binary subset of classes 
        this = binaryTrain(this,trainPatterns,trainLabels)
        % returns copy for a binary classifier
        new = binaryCopy(this)
    end
    
    methods
        
        function jc = train(jc,trainPatterns,trainLabels)
            % splits up multiple classes in binary classification problems
            % and calls correspondending binary classifiers
            % (binaryTrain/binaryTest)
            obs = unique(trainLabels);
            % 2 classes -> use native method
            if numel(obs) < 3
                jc = binaryTrain(jc,trainPatterns,trainLabels);
                jc.isBinary = 1;
            else
                jc.isBinary = 0;
                jc.classifierStruct.obs = obs;
                switch jc.multiClassMethod
                    case 'native'
                        jc = binaryTrain(jc,trainPatterns,trainLabels);
                    case 'onevsone'
                        counter = 1;
                        for k=1:numel(obs)
                            for h=k+1:numel(obs)
                                tmpLabels = trainLabels;
                                tmpPatterns = trainPatterns;
                                tmpLabels(tmpLabels ~= obs(k) & tmpLabels ~= obs(h)) =nan;
                                tmpPatterns(:,isnan(tmpLabels)) = [];
                                tmpLabels(isnan(tmpLabels)) = [];
                                morphed{counter} = tmpLabels;
                                jc.classifierStruct.morphed = morphed;
                                jc.classifierStruct.classifiers{counter} = jc.binaryCopy;
                                jc.classifierStruct.classifiers{counter} = jc.classifierStruct.classifiers{counter}.binaryTrain(tmpPatterns,morphed{counter});
                                counter = counter  + 1;
                            end
                        end
                    case 'onevsall'
                        for k=1:numel(obs)
                            tmpLabels = trainLabels;
                            tmpLabels(tmpLabels == obs(k)) = NaN;
                            tmpLabels(~isnan(tmpLabels)) = 1;
                            tmpLabels(isnan(tmpLabels)) = 0;
                            morphed{k} = tmpLabels;
                            jc.classifierStruct.morphed = morphed;
                            jc.classifierStruct.classifiers{k} = jc.binaryCopy;
                            jc.classifierStruct.classifiers{k} = jc.classifierStruct.classifiers{k}.binaryTrain(trainPatterns,morphed{k});
                        end
                    otherwise
                        throw(MException('janis:UnkownMethod','MultiClass method is unkown.'));
                end
            end
        end
        
        function [predictions decFactors] = test(jc,testPatterns)
            if jc.isBinary || strcmp(jc.multiClassMethod,'native')
                [predictions, decFactors] = jc.binaryTest(testPatterns);
            else
                switch jc.multiClassMethod
                    case 'onevsone'
                        preds = [];
                        obs = jc.classifierStruct.obs;
                        for k=1:length(obs)
                            [singlePred, decFactors] = jc.classifierStruct.classifiers{k}.binaryTest(testPatterns);
                            preds = [preds singlePred];
                        end
                        [vals, pos, freq] = mode(preds,2);
                        if max(cellfun(@numel,freq)) > 1
                            throw(MException('janis:PattDecision','This classifier does not support decision values - therefore patt situation cannot be handeled.'));
                        end
                        predictions = vals;
                        decFactors.decVals = [];
                    case 'onevsall'
                        vals = [];
                        obs = jc.classifierStruct.obs;
                        for k=1:length(obs)
                            [singlePred, decFactors] = jc.classifierStruct.classifiers{k}.binaryTest(testPatterns);
                            if ~isfield(decFactors,'decVals')
                                % throw error
                            end
                            vals = [vals decFactors.decVals];
                        end
                        [maxVal maxLoc] = max(vals,[],2);
                        predictions = obs(maxLoc);
                        decFactors.decVals = maxVal;
                    otherwise
                        throw(MException('janis:UnkownMethod','MultiClass method is unkown.'));
                end
            end
        end
        
        function show(jc)
            show@janis_classifier(jc);
            if ~isempty(jc.isBinary) && ~strcmp(jc.multiClassMethod,'native')
                if jc.isBinary
                    fprintf('Binary classification\n');
                else
                    fprintf('Non-Binary -> Using method %s\n',jc.multiClassMethod);
                end
            end
        end
    end
    
end
